@@ -61,6 +61,7 @@ struct sink {
 	struct avdtp_stream *stream;
 	unsigned int cb_id;
 	guint dc_id;
+	guint retry_id;
 	avdtp_session_state_t session_state;
 	avdtp_state_t stream_state;
 	sink_state_t state;
@@ -138,8 +139,11 @@ static void avdtp_state_callback(struct audio_device *dev,
 			emit_property_changed(dev->conn, dev->path,
 					AUDIO_SINK_INTERFACE, "Connected",
 					DBUS_TYPE_BOOLEAN, &value);
-			device_remove_disconnect_watch(dev->btd_dev,
-							sink->dc_id);
+			if (sink->dc_id) {
+				device_remove_disconnect_watch(dev->btd_dev,
+								sink->dc_id);
+				sink->dc_id = 0;
+			}
 		}
 		sink_set_state(dev, SINK_STATE_DISCONNECTED);
 		break;
@@ -166,7 +170,8 @@ static void pending_request_free(struct audio_device *dev,
 	g_free(pending);
 }
 
-static void disconnect_cb(struct btd_device *btd_dev, void *user_data)
+static void disconnect_cb(struct btd_device *btd_dev, gboolean removal,
+				void *user_data)
 {
 	struct audio_device *device = user_data;
 	struct sink *sink = device->sink;
@@ -203,6 +208,12 @@ static void stream_state_changed(struct avdtp_stream *stream,
 			pending_request_free(dev, p);
 		}
 
+		if (sink->dc_id) {
+			device_remove_disconnect_watch(dev->btd_dev,
+							sink->dc_id);
+			sink->dc_id = 0;
+		}
+
 		if (sink->session) {
 			avdtp_unref(sink->session);
 			sink->session = NULL;
@@ -268,6 +279,8 @@ static gboolean stream_setup_retry(gpointer user_data)
 	struct sink *sink = user_data;
 	struct pending_request *pending = sink->connect;
 
+	sink->retry_id = 0;
+
 	if (sink->stream_state >= AVDTP_STATE_OPEN) {
 		debug("Stream successfully created, after XCASE connect:connect");
 		if (pending->msg) {
@@ -318,8 +331,9 @@ static void stream_setup_complete(struct avdtp *session, struct a2dp_sep *sep,
 	if (avdtp_error_type(err) == AVDTP_ERROR_ERRNO
 			&& avdtp_error_posix_errno(err) != EHOSTDOWN) {
 		debug("connect:connect XCASE detected");
-		g_timeout_add_seconds(STREAM_SETUP_RETRY_TIMER,
-				stream_setup_retry, sink);
+		sink->retry_id = g_timeout_add_seconds(STREAM_SETUP_RETRY_TIMER,
+							stream_setup_retry,
+							sink);
 	} else {
 		if (pending->msg)
 			error_failed(pending->conn, pending->msg, "Stream setup failed");
@@ -484,8 +498,10 @@ static void discovery_complete(struct avdtp *session, GSList *seps, struct avdtp
 		if (avdtp_error_type(err) == AVDTP_ERROR_ERRNO
 				&& avdtp_error_posix_errno(err) != EHOSTDOWN) {
 			debug("connect:connect XCASE detected");
-			g_timeout_add_seconds(STREAM_SETUP_RETRY_TIMER,
-					stream_setup_retry, sink);
+			sink->retry_id =
+				g_timeout_add_seconds(STREAM_SETUP_RETRY_TIMER,
+							stream_setup_retry,
+							sink);
 		} else
 			goto failed;
 		return;
@@ -715,6 +731,9 @@ static void sink_free(struct audio_device *dev)
 		avdtp_stream_remove_cb(sink->session, sink->stream,
 					sink->cb_id);
 
+	if (sink->dc_id)
+		device_remove_disconnect_watch(dev->btd_dev, sink->dc_id);
+
 	if (sink->session)
 		avdtp_unref(sink->session);
 
@@ -724,6 +743,9 @@ static void sink_free(struct audio_device *dev)
 	if (sink->disconnect)
 		pending_request_free(dev, sink->disconnect);
 
+	if (sink->retry_id)
+		g_source_remove(sink->retry_id);
+
 	g_free(sink);
 	dev->sink = NULL;
 }
