@@ -59,6 +59,7 @@ struct main_opts {
 	uint16_t	link_policy;
 	gboolean	remember_powered;
 	gboolean	reverse_sdp;
+	gboolean	name_resolv;
 
 	uint8_t		scan;
 	uint8_t		mode;
@@ -84,6 +85,9 @@ void hci_req_queue_remove(int dev_id, bdaddr_t *dba);
 void start_security_manager(int hdev);
 void stop_security_manager(int hdev);
 
+void btd_start_exit_timer(void);
+void btd_stop_exit_timer(void);
+
 void set_pin_length(bdaddr_t *sba, int length);
 
 gboolean plugin_init(GKeyFile *config);
